/*
Copyright © 2022 Lukas Bachschwell <lukas@lbsfilm.at>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package cmd

import (
	"os"
	"path/filepath"
	"strings"

	log "github.com/s00500/env_logger"
	"github.com/spf13/cobra"
)

// hostnameCmd represents the hostname command
var hostnameCmd = &cobra.Command{
	Use:   "hostname",
	Short: "Change the hostname of the system easily",
	Long:  `Change the hostname of the system easily`,
	Args:  cobra.ExactArgs(1),
	Run: func(cmd *cobra.Command, args []string) {
		hostname := args[0]
		log.Println("changing hostname to ", hostname)

		prepareWorkdir()

		// read current
		fileDat, err := os.ReadFile(filepath.Join(workdir, "etc", "hostname"))
		log.MustFatal(err)
		oldName := strings.TrimSpace(string(fileDat))
		if oldName == "" {
			log.Fatal("Could not get old hostname")
		}

		// Write new Hostnbame
		err = os.WriteFile(filepath.Join(workdir, "etc", "hostname"), []byte(hostname), 0755)
		log.MustFatal(err)

		// Finally replace all in host file
		hostsDat, err := os.ReadFile(filepath.Join(workdir, "etc", "hosts"))
		log.MustFatal(err)

		hostsNew := strings.ReplaceAll(string(hostsDat), oldName, hostname)

		err = os.WriteFile(filepath.Join(workdir, "etc", "hosts"), []byte(hostsNew), 0755)
		log.MustFatal(err)

		createOutput()
	},
}

func init() {
	rootCmd.AddCommand(hostnameCmd)

	// Here you will define your flags and configuration settings.

	// Cobra supports Persistent Flags which will work for this command
	// and all subcommands, e.g.:
	// hostnameCmd.PersistentFlags().String("foo", "", "A help for foo")

	// Cobra supports local flags which will only run when this command
	// is called directly, e.g.:
	// hostnameCmd.Flags().BoolP("toggle", "t", false, "Help message for toggle")
}
