/*
Copyright © 2022 Lukas Bachschwell <lukas@lbsfilm.at>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package cmd

import (
	"os"
	"os/user"
	"path/filepath"
	"strings"

	log "github.com/s00500/env_logger"
	"github.com/spf13/cobra"
)

var keyFile string

// keyCmd represents the key command
var keyCmd = &cobra.Command{
	Use:   "key",
	Short: "A brief description of your command",
	Long: `A longer description that spans multiple lines and likely contains examples
and usage of using your command. For example:

Cobra is a CLI library for Go that empowers applications.
This application is a tool to generate the needed files
to quickly create a Cobra application.`,
	Run: func(cmd *cobra.Command, args []string) {
		log.Info("key called")

		// promt for a user
		userName := "root"
		if len(args) > 0 {
			userName = args[0]
		}

		if strings.Contains(keyFile, "~") {

			usr, err := user.Current()
			if err != nil {
				log.Fatal(err)
			}
			keyFile = strings.Replace(keyFile, "~", usr.HomeDir, 1)
		}

		keyDat, err := os.ReadFile(keyFile)
		log.MustFatal(err)

		prepareWorkdir()

		path := workdir + "/home/" + userName + "/.ssh"
		if userName == "root" {
			path = workdir + "/root/.ssh"
		}
		err = os.MkdirAll(path, 0755)
		log.MustFatal(err)

		f, err := os.OpenFile(filepath.Join(path, "authorized_keys"), os.O_APPEND|os.O_CREATE|os.O_WRONLY, 0644)
		log.MustFatal(err)

		defer f.Close()
		_, err = f.WriteString(string(keyDat) + "\n")
		log.MustFatal(err)

		createOutput()
	},
}

func init() {
	sshCmd.AddCommand(keyCmd)
	keyCmd.Flags().StringVar(&keyFile, "t", "~/.ssh/id_rsa.pub", "Public Key File to insert into overlay")
}
